// Copyright Varjo Technologies Oy. All Rights Reserved.

#pragma once

#include "IEyeTrackerModule.h"
#include "EyeTrackerTypes.h"
#include "IEyeTracker.h"
#include "Modules/ModuleManager.h"
#include "GameFramework/HUD.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "VarjoEyeTrackerTypes.h"
#include "VarjoEyeTracker.h"
#include "VarjoGaze.h"

class IVarjoEyeTrackerModule : public IEyeTrackerModule
{
public:
	static inline IVarjoEyeTrackerModule& Get()
	{
		return FModuleManager::LoadModuleChecked<IVarjoEyeTrackerModule>("VarjoEyeTracker");
	}

	static inline bool IsAvailable()
	{
		return FModuleManager::Get().IsModuleLoaded("VarjoEyeTracker");
	}

	virtual FString GetModuleKeyName() const override
	{
		return TEXT("VarjoEyeTracker");
	};
};

class FVarjoEyeTrackerModule : public IVarjoEyeTrackerModule
{
public:
	virtual void StartupModule() override;
	virtual void ShutdownModule() override;
	virtual TSharedPtr< class IEyeTracker, ESPMode::ThreadSafe > CreateEyeTracker() override;
	virtual bool IsEyeTrackerConnected() const override;
	TSharedPtr<FVarjoEyeTracker, ESPMode::ThreadSafe> VarjoEyeTracker;

private:
	FDelegateHandle OnDrawDebugHandle;
	FDelegateHandle OnPreLoadMapHandle;

	void OnDrawDebug(AHUD* HUD, UCanvas* Canvas, const FDebugDisplayInfo& DisplayInfo, float& YL, float& YPos);
};
