// Copyright Varjo Technologies Oy. All Rights Reserved.

#pragma once

#include "Kismet/BlueprintFunctionLibrary.h"
#include "varjo_types.h"
#include "VarjoEyeTrackerTypes.generated.h"

USTRUCT(BlueprintType)
struct FVarjoEyeTrackingRay
{
	GENERATED_USTRUCT_BODY()

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	FVector start;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	FVector dir;
};

USTRUCT(BlueprintType)
struct FVarjoEyeTrackingData
{
	GENERATED_USTRUCT_BODY()

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	FVarjoEyeTrackingRay leftEye;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	FVarjoEyeTrackingRay rightEye;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	FVarjoEyeTrackingRay gaze;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	float focusDistance;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	float stability;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	int captureTime;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	int leftStatus;

	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	int rightStatus;

	UPROPERTY(BlueprintReadWrite, Category = "Varjo Data")
	int status;
	
	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	int frameNumber;
	
	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	float leftPupilSize;
	
	UPROPERTY(BlueprintReadOnly, Category = "Varjo Data")
	float rightPupilSize;

	UPROPERTY(BlueprintReadWrite, Category = "Varjo Data")
	int targetId;

	UPROPERTY(BlueprintReadWrite, Category = "Varjo Data")
	FVector targetPos;

	UPROPERTY(BlueprintReadWrite, Category = "Varjo Data")
	float targetDist;

	varjo_Gaze rawData;
};

UENUM(BlueprintType)
enum class EVarjoEyeTrackingStatus : uint8
{
	NotConnected					UMETA(DisplayName = "Not Connected"),
	Disabled						UMETA(DisplayName = "Disabled"),
	NotCalibrated					UMETA(DisplayName = "Not Calibrated"),
	Calibrated						UMETA(DisplayName = "Calibrated"),
};

/** Possible calibration statuses the eye tracker could report. */
UENUM(BlueprintType)
enum class EVarjoEyeTrackingCalibrationStatus : uint8
{
	None,
	Bad,
	Good,
};
