// Copyright 2019 Varjo Technologies Oy. All rights reserved.

#ifndef VARJO_TYPES_H
#define VARJO_TYPES_H

#include <stdint.h>

#if defined __cplusplus
extern "C" {
#endif

typedef int64_t varjo_Error;
static const varjo_Error varjo_Error_InvalidSession = 1;
static const varjo_Error varjo_Error_OutOfMemory = 2;
static const varjo_Error varjo_Error_InvalidVersion = 3;
static const varjo_Error varjo_Error_GraphicsNotInitialized = 4;
static const varjo_Error varjo_Error_FrameNotStarted = 5;
static const varjo_Error varjo_Error_FrameAlreadyStarted = 6;
static const varjo_Error varjo_Error_ViewIndexOutOfBounds = 7;
static const varjo_Error varjo_Error_InvalidPoseType = 8;
static const varjo_Error varjo_Error_NullPointer = 9;
static const varjo_Error varjo_Error_MixingTextures = 10;
static const varjo_Error varjo_Error_NaN = 11;
static const varjo_Error varjo_Error_NoHMDConnected = 12;

// Gaze errors (100 - 199)
static const varjo_Error varjo_Error_GazeNotInitialized = 100;
static const varjo_Error varjo_Error_GazeNotConnected = 101;
static const varjo_Error varjo_Error_GazeAlreadyInitialized = 102;
static const varjo_Error varjo_Error_GazeNotAllowed = 103;

// Graphics errors (200 - 299)
static const varjo_Error varjo_Error_D3D11DeviceCreationFailed = 200;
static const varjo_Error varjo_Error_GetD3D11_1DeviceFailed = 201;
static const varjo_Error varjo_Error_AcquireD3D11DeviceHandleFailed = 202;
static const varjo_Error varjo_Error_GLBackBufferAlreadyCreated = 203;
static const varjo_Error varjo_Error_GLExtensionNotFound = 204;
static const varjo_Error varjo_Error_GLCreateTextureFailed = 205;
static const varjo_Error varjo_Error_GLAcquireD3D11TextureHandleFailed = 206;
static const varjo_Error varjo_Error_GLLockTextureFailed = 207;
static const varjo_Error varjo_Error_GLCopyTextureFailed = 208;
static const varjo_Error varjo_Error_OpenSharedTextureFailed = 209;
static const varjo_Error varjo_Error_TextureMutexQueryFailed = 210;
static const varjo_Error varjo_Error_TextureMutexAcquireFailed = 211;
static const varjo_Error varjo_Error_TextureMutexReleaseFailed = 212;
static const varjo_Error varjo_Error_D3D11AlreadyInitialized = 213;
static const varjo_Error varjo_Error_D3D11ShutDownFailed = 214;
static const varjo_Error varjo_Error_GLAlreadyInitialized = 215;
static const varjo_Error varjo_Error_GLShutDownFailed = 216;
static const varjo_Error varjo_Error_UnsupportedTextureFormat = 217;
static const varjo_Error varjo_Error_SwapChainConfigInvalidWidth = 219;
static const varjo_Error varjo_Error_SwapChainConfigInvalidHeight = 220;
static const varjo_Error varjo_Error_SwapChainConfigInvalidTextureCount = 221;
static const varjo_Error varjo_Error_GraphicsShutDownFailed = 222;
static const varjo_Error varjo_Error_TextureIsNull = 223;
static const varjo_Error varjo_Error_InvalidFrameNumber = 224;
static const varjo_Error varjo_Error_InvalidRenderAPI = 225;
static const varjo_Error varjo_Error_InvalidMatrix = 226;
static const varjo_Error varjo_Error_UnsupportedDepthFormat = 227;
static const varjo_Error varjo_Error_InvalidClipDistances = 229;
static const varjo_Error varjo_Error_SwapChainInvalidTextureIndex = 230;

// Compositor errors (300 - 399)
static const varjo_Error varjo_Error_ConnectToProgramManagerFailed = 300;
static const varjo_Error varjo_Error_RequestSwapChainFormatFailed = 301;
static const varjo_Error varjo_Error_OpenSharedEventFailed = 302;
static const varjo_Error varjo_Error_CreateIPCFailed = 303;
static const varjo_Error varjo_Error_InitializeCompositorLinkFailed = 304;
static const varjo_Error varjo_Error_RequestViewInfoFailed = 305;
static const varjo_Error varjo_Error_NoCompositorLink = 306;
static const varjo_Error varjo_Error_RequestCreateSwapChainFailed = 307;

// Special error codes
static const varjo_Error varjo_NoError = 0;
static const varjo_Error varjo_Error_Unknown = -1;

typedef int32_t varjo_Bool;
static const varjo_Bool varjo_False = 0;
static const varjo_Bool varjo_True = 1;

/**
 * Time in nanoseconds. Absolute times are relative to an epoch which is constant during execution of the program.
 */
typedef int64_t varjo_Nanoseconds;

/**
 * Gaze statuses.
 */
typedef int64_t varjo_GazeStatus;
static const varjo_GazeStatus varjo_GazeStatus_Invalid = 0;  //!< Data is not available, user is not wearing the device or eyes can not be found.
static const varjo_GazeStatus varjo_GazeStatus_Adjust = 1;   //!< User is wearing the device but gaze tracking is in middle of adjustment.
static const varjo_GazeStatus varjo_GazeStatus_Valid = 2;    //!< Data is valid.

typedef int64_t varjo_GazeEyeStatus;
static const varjo_GazeEyeStatus varjo_GazeEyeStatus_Invalid = 0;      //!< Eye is not tracked. (e.g. not visible or is shut).
static const varjo_GazeEyeStatus varjo_GazeEyeStatus_Visible = 1;      //!< Eye is visible but not reliably tracked (e.g. saccade or blink).
static const varjo_GazeEyeStatus varjo_GazeEyeStatus_Compensated = 2;  //!< Eye is tracked but quality compromised (e.g. headset has moved after calibration).
static const varjo_GazeEyeStatus varjo_GazeEyeStatus_Tracked = 3;      //!< Eye is tracked.

/**
 * Rotation reset types.
 */
typedef int64_t varjo_RotationReset;
static const varjo_RotationReset varjo_RotationReset_None = 0;  //!< Rotation is not reset.
static const varjo_RotationReset varjo_RotationReset_Yaw = 1;   //!< Yaw rotation (around up Y axis) is reset.
static const varjo_RotationReset varjo_RotationReset_All = 7;   //!< All rotation axes are reset.

/**
 * System status properties.
 */
typedef int64_t varjo_PropertyKey;
static const varjo_PropertyKey varjo_PropertyKey_Invalid = 0x0;                    //!< Invalid property key.
static const varjo_PropertyKey varjo_PropertyKey_GazeCalibrating = 0xA000;         //!< boolean. Is system currently calibrating the HMD gaze tracker.
static const varjo_PropertyKey varjo_PropertyKey_GazeCalibrated = 0xA001;          //!< boolean. Is the HMD gaze tracker calibrated.
static const varjo_PropertyKey varjo_PropertyKey_GazeCalibrationQuality = 0xA002;  //!< float [0.0-1.0]. Quality of the gaze calibration.
static const varjo_PropertyKey varjo_PropertyKey_GazeAllowed = 0xA003;             //!< boolean. Is the HMD gaze tracker allowed.
static const varjo_PropertyKey varjo_PropertyKey_GazeIPDEstimate = 0xA010;         //!< float. IPD distance estimate in millimeters.

typedef int64_t varjo_PoseType;
static const varjo_PoseType varjo_PoseType_LeftEye = 0x1;   //!< Pose for the left eye.
static const varjo_PoseType varjo_PoseType_Center = 0x2;    //!< Pose for the head (in the middle of the eyes).
static const varjo_PoseType varjo_PoseType_RightEye = 0x3;  //!< Pose for the right eye.

typedef int64_t varjo_DisplayType;
static const varjo_DisplayType varjo_DisplayType_Focus = 0x1;    //!< Focus display.
static const varjo_DisplayType varjo_DisplayType_Context = 0x2;  //!< Context display.

typedef int64_t varjo_Eye;
static const varjo_Eye varjo_Eye_Left = 0x1;   //!< Left eye.
static const varjo_Eye varjo_Eye_Right = 0x2;  //!< Right eye.

typedef int64_t varjo_WindingOrder;
static const varjo_WindingOrder varjo_WindingOrder_Clockwise = 0x1;         //!< Clockwise triangle winding.
static const varjo_WindingOrder varjo_WindingOrder_CounterClockwise = 0x2;  //!< Counter-clockwise triangle winding.

typedef int64_t varjo_TextureFormat;
static const varjo_TextureFormat varjo_TextureFormat_R8G8B8A8_SRGB = 0x1;  //!< sRgb 8-bit RGBA format
static const varjo_TextureFormat varjo_TextureFormat_R8B8G8A8_SRGB = 0x1;  //!< sRgb 8-bit RGBA format (deprecated due to a typo.)
static const varjo_TextureFormat varjo_TextureFormat_B8G8R8A8_SRGB = 0x2;  //!< sRgb 8-bit BGRA format

//!< 32-bit floating point depth format
static const varjo_TextureFormat varjo_DepthTextureFormat_D32_FLOAT = 0x3;

/**
 * Render API flags
 */
typedef int64_t varjo_RenderAPI;
static const varjo_RenderAPI varjo_RenderAPI_D3D11 = 0x1;  //!< Direct3D11 rendering API
static const varjo_RenderAPI varjo_RenderAPI_GL = 0x2;     //!< OpenGL rendering API

typedef int64_t varjo_EulerOrder;
static const varjo_EulerOrder varjo_EulerOrder_XYZ = 0;
static const varjo_EulerOrder varjo_EulerOrder_ZYX = 1;
static const varjo_EulerOrder varjo_EulerOrder_XZY = 2;
static const varjo_EulerOrder varjo_EulerOrder_YZX = 3;
static const varjo_EulerOrder varjo_EulerOrder_YXZ = 4;
static const varjo_EulerOrder varjo_EulerOrder_ZXY = 5;

typedef int64_t varjo_RotationDirection;
static const varjo_RotationDirection varjo_RotationDirection_Clockwise = -1;
static const varjo_RotationDirection varjo_RotationDirection_CounterClockwise = 1;

typedef int64_t varjo_Handedness;
static const varjo_Handedness varjo_Handedness_RightHanded = 1;
static const varjo_Handedness varjo_Handedness_LeftHanded = -1;

/**
 * Submit flags.
 */
typedef int64_t varjo_SubmitFlags;
static const int64_t varjo_SubmitFlag_Async = 1;        //!< Makes #varjo_EndFrame return immediately unless there is already a submittion in progress.
static const int64_t varjo_SubmitFlag_Opaque = 2;       //!< Frame is opaque and doesn't need blending (e.g. with VST).
static const int64_t varjo_SubmitFlag_InvertAlpha = 4;  //!< Invert alpha channel before blending.

struct varjo_Session;

/**
 * Double precision 4x4 matrix.
 *
 * The matrix usage convention is that they are stored in column-major order and transforms are stacked before
 * column-vector points when multiplying. That is, a pure translation matrix will have the position offset in elements 12..14.
 *
 * Unless otherwise specified, the coordinate system is right-handed: X goes right, Y goes up and negative Z goes forward.
 *
 */
struct varjo_Matrix {
    double value[16];
};

/**
 * Ray is a vector starting from an origin.
 */
struct varjo_Ray {
    double origin[3];   //!< Origin of the ray.
    double forward[3];  //!< Direction of the ray.
};

/**
 * 32bit floating point 2D vector.
 */
struct varjo_Vector2Df {
    float x;  //!< X coordinate.
    float y;  //!< Y coordinate.
};

/*
 * 64bit floating point 3D vector.
 */
struct varjo_Vector3D {
    double x;  //!< X coordinate.
    double y;  //!< Y coordinate.
    double z;  //!< Z coordinate.
};

/**
 * 2D triangle list mesh.
 */
struct varjo_Mesh2Df {
    struct varjo_Vector2Df* vertices;  //!< Vertex data.
    int32_t vertexCount;               //!< Number of vertices.
};

/**
 * Axis aligned tangents from a projection matrix.
 * The tangents are between the edge planes and projection centre and defined so that if
 * the projection is centered, all of them are positive.
 */
struct varjo_AlignedView {
    double projectionTop;     //!< Tangent of the top edge angle.
    double projectionBottom;  //!< Tangent of the bottom edge angle.
    double projectionLeft;    //!< Tangent of the left edge angle.
    double projectionRight;   //!< Tangent of the right edge angle.
};

/**
 * View information for a frame.
 */
struct varjo_ViewInfo {
    double projectionMatrix[16];  //!< The projection matrix. See #varjo_Matrix for conventions.
    double viewMatrix[16];        //!< The view matrix, world-to-eye. See #varjo_Matrix for conventions.
    int32_t preferredWidth;       //!< Preferred width of the viewport.
    int32_t preferredHeight;      //!< Preferred height of the viewport.
    varjo_Bool enabled;           //!< Whether this view should be rendered during current frame.
    int32_t reserved;             //!< Unused.
};

/**
 * Per-frame information.
 */
struct varjo_FrameInfo {
    struct varjo_ViewInfo* views;   //!< Views for the frame. Array size is #varjo_GraphicsInfo::viewCount.
    varjo_Nanoseconds displayTime;  //!< When the frame is estimated to be displayed.
    int64_t frameNumber;            //!< Current frame number.
};

/**
 * View description.
 * Information about the associated eye, display and resolution.
 */
struct varjo_ViewDescription {
    int32_t width;              //!< Default view width in pixels.
    int32_t height;             //!< Default view height in pixels.
    varjo_DisplayType display;  //!< Which display the view is for.
    varjo_Eye eye;              //!< Which eye the view is for.
};

/**
 * Gaze tracker vectors and tracking state.
 */
struct varjo_Gaze {
    struct varjo_Ray leftEye;         //!< Left eye gaze ray.
    struct varjo_Ray rightEye;        //!< Right eye gaze ray.
    struct varjo_Ray gaze;            //!< Normalized gaze direction ray.
    double focusDistance;             //!< Estimated gaze direction focus point distance.
    double stability;                 //!< Focus point stability.
    varjo_Nanoseconds captureTime;    //!< Varjo time when this data was captured, see varjo_GetCurrentTime()
    varjo_GazeEyeStatus leftStatus;   //!< Status of left eye data.
    varjo_GazeEyeStatus rightStatus;  //!< Status of right eye data.
    varjo_GazeStatus status;          //!< Tracking main status.
    int64_t frameNumber;              //!< Frame number, increases monotonically.
    double leftPupilSize;             //!< Normalized [0..1] left eye pupil size.
    double rightPupilSize;            //!< Normalized [0..1] right eye pupil size.
};

/**
 * Config for swap chain texture size and count.
 */
struct varjo_SwapChainConfig {
    int32_t numberOfTextures;  //!< Number of swap chain textures.
    int32_t textureWidth;      //!< Texture width.
    int32_t textureHeight;     //!< Texture height.
    int32_t reserved;          //!< Unused.
};

/**
 * Minimum and maximum limits for swap chain texture count and size.
 */
struct varjo_SwapChainLimits {
    int32_t minimumNumberOfTextures;  //!< Minimum number of swap chain textures.
    int32_t maximumNumberOfTextures;  //!< Maximum number of swap chain textures.
    int32_t minimumTextureWidth;      //!< Minimum width of the swap chain textures.
    int32_t minimumTextureHeight;     //!< Minimum height of the swap chain textures.
    int32_t maximumTextureWidth;      //!< Maximum width of the swap chain textures.
    int32_t maximumTextureHeight;     //!< Maximum height of the swap chain textures.
};

/**
 * Viewport rectangle that defines viewport area.
 */
struct varjo_Viewport {
    int32_t x;       //!< X coordinate for the view in pixels.
    int32_t y;       //!< Y coordinate for the view in pixels.
    int32_t width;   //!< Width of the view in pixels.
    int32_t height;  //!< Height of the view in pixels.
};

/**
 * General graphics information.
 */
struct varjo_GraphicsInfo {
    struct varjo_Texture* swapChainTextures;  //!< Swap chain textures.
    int32_t swapChainTextureCount;            //!< Number of swap chain textures.
    int32_t swapChainTextureWidth;            //!< Swap chain texture width.
    int32_t swapChainTextureHeight;           //!< Swap chain texture height.
    int32_t viewCount;                        //!< Number of views.
};

/**
 * API-agnostic texture handle.
 * Use #varjo_FromD3D11Texture, varjo_ToD3D11Texture
 * #varjo_FromGLTexture and #varjo_ToGLTexture
 * to convert textures between Varjo and graphics APIs.
 */
struct varjo_Texture {
    int64_t reserved[4];
};

/**
 * Information required to submit a frame to the Compositor.
 */
struct varjo_SubmitInfo {
    struct varjo_Viewport* viewports;  //!< Viewports to submit.
    struct varjo_Texture* textures;    //!< Textures to submit.
    varjo_SubmitFlags flags;           //!< Additional submit flags.
    int64_t frameNumber;               //!< Current frame number.
};

#if defined __cplusplus
}
#endif

#endif  // VARJO_TYPES_H
